@@ -443,8 +443,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  * - some overlay fields added
  * - what does it all mean?
  */
-#define FWATER_BLC       0x20d8
-#define FWATER_BLC2	 0x20dc
+#define FWATER_BLC1         0x20D8
+#define FWATER_BLC2         0x20DC
+#define FWATER_BLC_SELF     0x20E0
+#define FWATER_BLC3         0x20EC
 #define MM_BURST_LENGTH     0x00700000
 #define MM_FIFO_WATERMARK   0x0001F000
 #define LM_BURST_LENGTH     0x00000700
@@ -818,55 +820,56 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #define DPLL_A		0x06014
 #define DPLL_B		0x06018
-# define DPLL_VCO_ENABLE			(1 << 31)
-# define DPLL_DVO_HIGH_SPEED			(1 << 30)
-# define DPLL_SYNCLOCK_ENABLE			(1 << 29)
-# define DPLL_VGA_MODE_DIS			(1 << 28)
-# define DPLLB_MODE_DAC_SERIAL			(1 << 26)	/* i915 */
-# define DPLLB_MODE_LVDS			(2 << 26)	/* i915 */
-# define DPLL_MODE_MASK				(3 << 26)
-# define DPLL_DAC_SERIAL_P2_CLOCK_DIV_10	(0 << 24)	/* i915 */
-# define DPLL_DAC_SERIAL_P2_CLOCK_DIV_5		(1 << 24)	/* i915 */
-# define DPLLB_LVDS_P2_CLOCK_DIV_14		(0 << 24)	/* i915 */
-# define DPLLB_LVDS_P2_CLOCK_DIV_7		(1 << 24)	/* i915 */
-# define DPLL_P2_CLOCK_DIV_MASK			0x03000000	/* i915 */
-# define DPLL_FPA01_P1_POST_DIV_MASK		0x00ff0000	/* i915 */
+#define DPLL_VCO_ENABLE			(1 << 31)
+#define DPLL_DVO_HIGH_SPEED			(1 << 30)
+#define DPLL_SYNCLOCK_ENABLE			(1 << 29)
+#define DPLL_VGA_MODE_DIS			(1 << 28)
+#define DPLLB_MODE_DAC_SERIAL			(1 << 26)	/* i915 */
+#define DPLLB_MODE_LVDS			(2 << 26)	/* i915 */
+#define DPLL_MODE_MASK				(3 << 26)
+#define DPLL_DAC_SERIAL_P2_CLOCK_DIV_10	(0 << 24)	/* i915 */
+#define DPLL_DAC_SERIAL_P2_CLOCK_DIV_5		(1 << 24)	/* i915 */
+#define DPLLB_LVDS_P2_CLOCK_DIV_14		(0 << 24)	/* i915 */
+#define DPLLB_LVDS_P2_CLOCK_DIV_7		(1 << 24)	/* i915 */
+#define DPLL_P2_CLOCK_DIV_MASK			0x03000000	/* i915 */
+#define DPLL_FPA01_P1_POST_DIV_MASK		0x00ff0000	/* i915 */
 /**
  *  The i830 generation, in DAC/serial mode, defines p1 as two plus this
  * bitfield, or just 2 if PLL_P1_DIVIDE_BY_TWO is set.
  */
-# define DPLL_FPA01_P1_POST_DIV_MASK_I830	0x001f0000
+#define DPLL_FPA01_P1_POST_DIV_MASK_I830	0x001f0000
 /**
  * The i830 generation, in LVDS mode, defines P1 as the bit number set within
  * this field (only one bit may be set).
  */
-# define DPLL_FPA01_P1_POST_DIV_MASK_I830_LVDS	0x003f0000
-# define DPLL_FPA01_P1_POST_DIV_SHIFT		16
-# define PLL_P2_DIVIDE_BY_4			(1 << 23)	/* i830, required in DVO non-gang */
-# define PLL_P1_DIVIDE_BY_TWO			(1 << 21)	/* i830 */
-# define PLL_REF_INPUT_DREFCLK			(0 << 13)
-# define PLL_REF_INPUT_TVCLKINA			(1 << 13)	/* i830 */
-# define PLL_REF_INPUT_TVCLKINBC		(2 << 13)	/* SDVO TVCLKIN */
-# define PLLB_REF_INPUT_SPREADSPECTRUMIN	(3 << 13)
-# define PLL_REF_INPUT_MASK			(3 << 13)
-# define PLL_LOAD_PULSE_PHASE_SHIFT		9
+#define DPLL_FPA01_P1_POST_DIV_MASK_I830_LVDS	0x003f0000
+#define DPLL_FPA01_P1_POST_DIV_SHIFT		16
+#define PLL_P2_DIVIDE_BY_4			(1 << 23)	/* i830, required in DVO non-gang */
+#define PLL_P1_DIVIDE_BY_TWO			(1 << 21)	/* i830 */
+#define PLL_REF_INPUT_DREFCLK			(0 << 13)
+#define PLL_REF_INPUT_TVCLKINA			(1 << 13)	/* i830 */
+#define PLL_REF_INPUT_TVCLKINBC		(2 << 13)	/* SDVO TVCLKIN */
+#define PLLB_REF_INPUT_SPREADSPECTRUMIN	(3 << 13)
+#define PLL_REF_INPUT_MASK			(3 << 13)
+#define PLL_LOAD_PULSE_PHASE_SHIFT		9
+#define DPLL_CLOCK_PHASE_9	(1<<9 | 1<<12)
 /*
  * Parallel to Serial Load Pulse phase selection.
  * Selects the phase for the 10X DPLL clock for the PCIe
  * digital display port. The range is 4 to 13; 10 or more
  * is just a flip delay. The default is 6
  */
-# define PLL_LOAD_PULSE_PHASE_MASK		(0xf << PLL_LOAD_PULSE_PHASE_SHIFT)
-# define DISPLAY_RATE_SELECT_FPA1		(1 << 8)
+#define PLL_LOAD_PULSE_PHASE_MASK		(0xf << PLL_LOAD_PULSE_PHASE_SHIFT)
+#define DISPLAY_RATE_SELECT_FPA1		(1 << 8)
 
 /**
  * SDVO multiplier for 945G/GM. Not used on 965.
  *
  * \sa DPLL_MD_UDI_MULTIPLIER_MASK
  */
-# define SDVO_MULTIPLIER_MASK			0x000000ff
-# define SDVO_MULTIPLIER_SHIFT_HIRES		4
-# define SDVO_MULTIPLIER_SHIFT_VGA		0
+#define SDVO_MULTIPLIER_MASK			0x000000ff
+#define SDVO_MULTIPLIER_SHIFT_HIRES		4
+#define SDVO_MULTIPLIER_SHIFT_VGA		0
 
 /** @defgroup DPLL_MD
  * @{
@@ -880,11 +883,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  * Value is pixels minus 1.  Must be set to 1 pixel for SDVO.
  */
-# define DPLL_MD_UDI_DIVIDER_MASK		0x3f000000
-# define DPLL_MD_UDI_DIVIDER_SHIFT		24
+#define DPLL_MD_UDI_DIVIDER_MASK		0x3f000000
+#define DPLL_MD_UDI_DIVIDER_SHIFT		24
 /** UDI pixel divider for VGA, same as DPLL_MD_UDI_DIVIDER_MASK. */
-# define DPLL_MD_VGA_UDI_DIVIDER_MASK		0x003f0000
-# define DPLL_MD_VGA_UDI_DIVIDER_SHIFT		16
+#define DPLL_MD_VGA_UDI_DIVIDER_MASK		0x003f0000
+#define DPLL_MD_VGA_UDI_DIVIDER_SHIFT		16
 /**
  * SDVO/UDI pixel multiplier.
  *
@@ -931,6 +934,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define RAMCLK_GATE_D		0x6210 /* CRL only */
 
 #define BLC_PWM_CTL		0x61254
+
+#define BLC_UNKNOWN_TYPE	0x00
+#define BLC_I2C_TYPE		0x01
+#define BLC_PWM_TYPE		0x02
+
 #define BACKLIGHT_MODULATION_FREQ_SHIFT		(17)
 /**
  * This is the most significant 15 bits of the number of backlight cycles in a
@@ -948,7 +956,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  * backlight modulation cycle.
  */
 #define BACKLIGHT_DUTY_CYCLE_SHIFT		(0)
+#define BACKLIGHT_PWM_CTL_SHIFT		        (16)
 #define BACKLIGHT_DUTY_CYCLE_MASK		(0xffff)
+#define BACKLIGHT_PWM_POLARITY_BIT_CLEAR        (0xfffe)
+
+#define BLC_PRECISION_FACTOR		100 /* To preserve decimal fractions during integer calculations*/
 
 #define BLM_CTL			0x61260
 #define BLM_THRESHOLD_0		0x61270
@@ -2272,4 +2284,24 @@ typedef enum
 #define PALETTE_A		0x0a000
 #define PALETTE_B		0x0a800
 
+#define WM_DOTCLOCK_DIVISOR		1000000
+#define BITS_PER_BYTE			8
+#define DSPARB_ADDRESS			0x70030
+#define MAX_FIFO_START			128
+#define MEMORY_QUEUE_SIZE		16
+#define SELF_REFRESH_EXIT_LATENCY	100
+#define BITS_PER_CACHE_LINE		512
+#define PCI_PORT5_REGISTER3		0xD0050300
+#define PCI_DEV0_FUNC0_MCR_WRITE        0xD0
+#define PCI_DEV0_FUNC0_MDR_READ         0xD4
+#define MEM_CTRL_CLK_133		133
+#define MEM_CTRL_CLK_100		100
+#define MAX_FIFO_A_QITEM_LATENCY	12
+#define MAX_FIFO_B_QITEM_LATENCY	10
+#define DUAL_DISP_A_QITEM_LATENCY	15
+#define WATERMARK_PRECISION		10000
+#define WATERMARK_ROUNDING		5000
+#define C_START_POSITION		126
+#define B_START_POSITION		63
+
 #endif /* _I810_REG_H */
