@@ -40,6 +40,8 @@
 #include "i810_reg.h"
 #include "psb_driver.h"
 #include "xf86Modes.h"
+#include "psb_lvds.h"
+#include "psb_overlay.h"
 
 typedef struct
 {
@@ -307,6 +309,203 @@ psbPipeSetBase(xf86CrtcPtr crtc, int x, int y)
     (void)PSB_READ32(dspbase);
 }
 
+static U32
+psbGetMemClock(void)
+{
+    //calculate memory access timing
+    PCITAG host = pciTag(0, 0, 0);
+    CARD32 clock;
+
+#if (XORG_VERSION_MINOR >= 5 || (XORG_VERSION_MINOR == 4 && XORG_VERSION_PATCH == 99))
+    struct pci_device dev;
+
+    dev.domain = PCI_DOM_FROM_TAG(host);
+    dev.bus = PCI_BUS_FROM_TAG(host);
+    dev.dev = PCI_DEV_FROM_TAG(host);
+    dev.func = PCI_FUNC_FROM_TAG(host);
+
+    pci_device_cfg_write_u32(&dev, PCI_PORT5_REGISTER3, PCI_DEV0_FUNC0_MCR_WRITE);
+    pci_device_cfg_read_u32(&dev, &clock, PCI_DEV0_FUNC0_MDR_READ);
+#else
+    pciWriteLong(host, PCI_DEV0_FUNC0_MDR_READ, 0x00C32004);
+    pciWriteLong(host, PCI_DEV0_FUNC0_MCR_WRITE, 0xE0033000);
+    pciWriteLong(host, PCI_DEV0_FUNC0_MCR_WRITE, PCI_PORT5_REGISTER3);
+    clock = pciReadLong(host, PCI_DEV0_FUNC0_MDR_READ);
+#endif
+
+    if ((clock & 0x80) >> 3)
+        return MEM_CTRL_CLK_133;
+    else
+        return MEM_CTRL_CLK_100;
+}
+
+static U32 
+psbCalculateWaterMark(U32 in_start, Watermark_Type whichWM, U32 bpp, CePlane plane, U32 pixelClock, BOOL duel)
+{
+	// watermark values
+	U32 queueSize = MEMORY_QUEUE_SIZE;
+	U32 queueItemLatency = 0;
+	U32 latencySRExittoData = 0;
+	U32 SRExitLatency = SELF_REFRESH_EXIT_LATENCY;
+	U32 pixelClk = pixelClock;
+	U32 pixelClk_MemContClkRatio = 0;
+	U32 bitsPerCacheLine = BITS_PER_CACHE_LINE;
+	U32 bitsPerPixel = bpp;
+	U32 pixelsPerCacheLine = 1;
+	U32 watermark = 0;
+	U32 data;
+
+	// Get the memory controller clock value
+	U32 memoryContClk = psbGetMemClock();
+
+	if (plane == DCplane_A)
+	{
+		queueItemLatency = MAX_FIFO_A_QITEM_LATENCY;
+		if (duel)
+		{
+			queueItemLatency = 15;
+		}
+	}
+	else
+	{
+		queueItemLatency = MAX_FIFO_B_QITEM_LATENCY;
+	}
+
+	latencySRExittoData = queueSize * queueItemLatency;
+	pixelClk_MemContClkRatio = (pixelClk * WATERMARK_PRECISION) / memoryContClk;
+	pixelsPerCacheLine = bitsPerCacheLine / bitsPerPixel;
+
+	switch (whichWM)
+	{
+		case WATERMARK_2:
+			watermark = in_start - (((latencySRExittoData * (pixelClk_MemContClkRatio/pixelsPerCacheLine)) + WATERMARK_ROUNDING) / WATERMARK_PRECISION);
+			break;
+		case WATERMARK_1:
+			watermark = in_start - (((SRExitLatency * (pixelClk_MemContClkRatio/pixelsPerCacheLine)) + WATERMARK_ROUNDING) / WATERMARK_PRECISION) - 
+						(((latencySRExittoData * (pixelClk_MemContClkRatio/pixelsPerCacheLine)) + WATERMARK_ROUNDING) / WATERMARK_PRECISION);
+			break;
+		default:
+			return 0;
+	}
+
+	return watermark;
+}
+
+static void
+psbSetWatermarks(ScrnInfoPtr pScrn)
+{
+	PsbDevicePtr pDevice = psbDevicePTR(psbPTR(pScrn));
+	xf86OutputPtr output;
+	xf86CrtcConfigPtr xf86_config = XF86_CRTC_CONFIG_PTR(pScrn);
+	
+	Bool bPlaneAEnabled = FALSE;
+	Bool bPlaneBEnabled = FALSE;
+	int BPPA = 0;
+	int BPPB = 0;
+	unsigned long DotClockA = 0;
+	unsigned long DotClockB = 0;
+	CARD32 PlaneACtrl = PSB_READ32(DSPACNTR);
+	CARD32 PlaneBCtrl = PSB_READ32(DSPBCNTR);
+	CARD32 DisplayARBReg = PSB_READ32(DSPARB);
+
+	if (PlaneACtrl & DISPLAY_PLANE_ENABLE)
+		bPlaneAEnabled = TRUE;
+
+	if (PlaneBCtrl & DISPLAY_PLANE_ENABLE)
+		bPlaneBEnabled = TRUE;
+
+	if (!bPlaneAEnabled && !bPlaneBEnabled)
+	{
+		// There isn't any plane enabled, so don't change the watermarks.
+		return;
+	}
+
+	int i;
+	for (i = 0; i < xf86_config->num_output; i++)
+	{
+		output = xf86_config->output[i];
+
+		//port LVDS0, pipe1, planeB
+		if (!strcmp(output->name, "LVDS0"))
+		{
+			if (bPlaneBEnabled)
+			{   
+				//current mode dot clock, HZ
+				DotClockB = output->crtc->mode.Clock * 1000 / WM_DOTCLOCK_DIVISOR;
+				BPPB = output->scrn->bitsPerPixel;
+			}
+		}
+		//port SDVO, pipe0, planeA
+		else
+		{
+			if (bPlaneAEnabled)
+			{
+				DotClockA = output->crtc->mode.Clock * 1000 / WM_DOTCLOCK_DIVISOR;
+				BPPA = output->scrn->bitsPerPixel;
+			}
+		}
+
+	}
+
+	// Set the watermarks for both plane A and B
+	if (bPlaneAEnabled && bPlaneBEnabled)
+	{
+		CARD32 planeAWatermark_2, planeAWatermark_1, planeBWatermark_2, planeBWatermark_1;
+
+		// read c_start and b_start from the DSPARB register
+		DSPARB_Register dsparb;
+
+		dsparb.entireRegister = PSB_READ32(DSPARB);
+
+		planeAWatermark_1 = psbCalculateWaterMark(dsparb.b_Start, WATERMARK_1, BPPA, DCplane_A, DotClockA, TRUE);
+		planeAWatermark_2 = psbCalculateWaterMark(dsparb.b_Start, WATERMARK_2, BPPA, DCplane_A, DotClockA, TRUE);
+		planeBWatermark_1 = psbCalculateWaterMark(dsparb.c_Start - dsparb.b_Start, WATERMARK_1, BPPB, DCplane_B, DotClockB, TRUE);
+		planeBWatermark_2 = psbCalculateWaterMark(dsparb.c_Start - dsparb.b_Start, WATERMARK_2, BPPB, DCplane_B, DotClockB, TRUE);
+
+		// Write the watermark 1 register for A and B
+		Watermark_Register1 wm1;
+		wm1.entireRegister = PSB_READ32(FWATER_BLC1);
+		wm1.DispA_WM1 = planeAWatermark_1;
+		wm1.DispB_WM1 = planeBWatermark_1;
+
+		PSB_WRITE32(FWATER_BLC1, wm1.entireRegister);
+
+		// Write the watermark 2 register for both A and B
+		FIFOWatermark_Register wm2;
+		wm2.entireRegister = PSB_READ32(FWATER_BLC_SELF);
+		wm2.DispAB_WM2 = planeAWatermark_2 > planeBWatermark_2 ? planeBWatermark_2 : planeAWatermark_2;
+
+		PSB_WRITE32(FWATER_BLC_SELF, wm2.entireRegister); 
+	}	
+	// Set the MaxFIFO watermark
+	else
+	{
+		U32 start = MAX_FIFO_START;
+		U32 maxFIFOWatermark1_status0, maxFIFOWatermark1_status1;
+
+		if (bPlaneAEnabled)
+		{
+			maxFIFOWatermark1_status0 = psbCalculateWaterMark(start, WATERMARK_2, BPPA, DCplane_A, DotClockA, FALSE);
+			maxFIFOWatermark1_status1 = psbCalculateWaterMark(start, WATERMARK_1, BPPA, DCplane_A, DotClockA, FALSE);
+		}
+		else
+		{
+			maxFIFOWatermark1_status0 = psbCalculateWaterMark(start, WATERMARK_2, BPPB, DCplane_B, DotClockB, FALSE);
+			maxFIFOWatermark1_status1 = psbCalculateWaterMark(start, WATERMARK_1, BPPB, DCplane_B, DotClockB, FALSE);
+		}
+		
+		// Write the MAX FIFO Watermark register
+		MAXFIFOWatermark_Register wmRegister;
+		wmRegister.entireRegister = PSB_READ32(FWATER_BLC3);
+		wmRegister.WM1_STATUS0 = maxFIFOWatermark1_status0;
+		wmRegister.WM1_STATUS1 = maxFIFOWatermark1_status1;
+
+		PSB_WRITE32(FWATER_BLC3, wmRegister.entireRegister);
+	}
+
+	return;
+}
+
 /**
  * Sets the power management mode of the pipe and plane.
  *
@@ -318,6 +517,7 @@ psbCrtcDpms(xf86CrtcPtr crtc, int mode)
 {
     ScrnInfoPtr pScrn = crtc->scrn;
     PsbDevicePtr pDevice = psbDevicePTR(psbPTR(pScrn));
+    PsbPtr pPsb = psbPTR(pScrn);
 
     PsbCrtcPrivatePtr pCrtc = crtc->driver_private;
     int pipe = pCrtc->pipe;
@@ -374,13 +574,20 @@ psbCrtcDpms(xf86CrtcPtr crtc, int mode)
 
 	psbCrtcLoadLut(crtc);
 
-	/* Give the overlay scaler a chance to enable if it's on this pipe */
-	//      psbCrtcDpms_video(crtc, TRUE);
+	/* Enable the Overlay Plane when LVDS is turned on, and will
+	   update the Overlay Plane when SDVO is turned on. This is
+	   because the SDVO and LVDS share the same framebuffer, and
+	   SDVO's mode change will affect the Overlay Plane's input.*/
+	if (pPsb->downScale ) { 
+		psb_dpms_overlay(crtc, TRUE);
+	}
 	break;
     case DPMSModeOff:
 	PSB_DEBUG(crtc->scrn->scrnIndex, 3, "Crtc DPMS Off\n");
-	/* Give the overlay scaler a chance to disable if it's on this pipe */
-	// psbCrtcDpms_video(crtc, FALSE);
+	/* Give the overlay scaler a chance to disable if it's on LVDS */
+	if (pPsb->downScale && pipe==1) {
+		psb_dpms_overlay(crtc, FALSE);
+	}
 
 	/* Disable display plane */
 	temp = PSB_READ32(dspcntr_reg);
@@ -410,6 +617,9 @@ psbCrtcDpms(xf86CrtcPtr crtc, int mode)
 	usleep(150);
 	break;
     }
+
+    psbWaitForVblank(pScrn);
+    psbSetWatermarks(pScrn);
 }
 
 static Bool
@@ -480,7 +690,6 @@ psbPanelFitterPipe(CARD32 pfitControl)
 }
 
 /* Adjust the mode clock, refer to the codes in Vista driver Jamesx */
-typedef unsigned long U32;
 typedef struct _PBDCPLLregisters
 {
     U32 m1:6;
@@ -671,6 +880,7 @@ PBDCDisplaySDVO_FindOptimalTimingSDVO(U32 in_dotclock, U32 ulPortMultiplier,
 				  out_pPLLregisters);
 }
 
+
 /**
  * Sets up registers for the given mode/adjusted_mode pair.
  *
