@@ -91,18 +91,6 @@ static int psbEntityIndex = -1;
 #define makedev(x,y)    ((dev_t)(((x) << 8) | (y)))
 #endif
 
-#if (XORG_VERSION_MINOR >= 5 || (XORG_VERSION_MINOR == 4 && XORG_VERSION_PATCH == 99))
-#ifndef PCI_DOM_MASK
-#define PCI_DOM_MASK 0x0ffu
-#endif
-#define PCI_DOMBUS_MASK (((PCI_DOM_MASK) << 8)| 0x0ffu)
-
-#define PCI_DOM_FROM_TAG(tag) (((tag)>>24) & (PCI_DOM_MASK))
-#define PCI_BUS_FROM_TAG(tag) (((tag) >> 16) & (PCI_DOMBUS_MASK))
-#define PCI_DEV_FROM_TAG(tag) (((tag) & 0x0000f800u) >> 11)
-#define PCI_FUNC_FROM_TAG(tag) (((tag) & 0x00000700u) >> 8)
-#endif
-
 /*
  * This contains the functions needed by the server after loading the
  * driver module.  It must be supplied, and gets added the driver list by
