@@ -87,7 +87,7 @@
 #define PSB_NAME         "PSB"
 #define PSB_DRIVER_NAME  "psb"
 #define PSB_MAJOR_VERSION 0
-#define PSB_MINOR_VERSION 29
+#define PSB_MINOR_VERSION 32
 #define PSB_PATCHLEVEL    0
 
 #define PCI_CHIP_PSB1  0x8108
@@ -104,6 +104,16 @@
 
 #if (XORG_VERSION_MINOR >= 5 || (XORG_VERSION_MINOR == 4 && XORG_VERSION_PATCH == 99))
 typedef struct pci_device *pciVideoPtr;
+
+#ifndef PCI_DOM_MASK
+#define PCI_DOM_MASK 0x0ffu
+#endif
+
+#define PCI_DOMBUS_MASK (((PCI_DOM_MASK) << 8) | 0x0ffu)
+#define PCI_DOM_FROM_TAG(tag) (((tag)>>24) & (PCI_DOM_MASK))
+#define PCI_BUS_FROM_TAG(tag) (((tag) >> 16) & (PCI_DOMBUS_MASK))
+#define PCI_DEV_FROM_TAG(tag) (((tag) & 0x0000f800u) >> 11)
+#define PCI_FUNC_FROM_TAG(tag) (((tag) & 0x00000700u) >> 8)
 #endif
 
 typedef enum _PsbLidState
@@ -757,6 +767,7 @@ typedef enum _PBDC_SDVO_HDMI_BUF_INDEX {
 
 typedef unsigned char  U8;
 typedef unsigned short U16;
+typedef unsigned long  U32;
 
 //	Standard timing identification
 typedef union _STD_TIMIMG {
@@ -1053,6 +1064,90 @@ typedef enum _PBDC_SDVO_HDMI_AUDIO_STATE{
 
 /******************** HDMI InfoFrame End *******************************/
 
+
+typedef enum _Watermark_Type
+{
+    WATERMARK_1,
+    WATERMARK_2
+} Watermark_Type;
+
+typedef struct _DSPARB_Register
+{
+    union
+    {
+        U32 entireRegister;
+        struct
+        {
+            U32 b_Start         : 7;
+            U32 c_Start         : 7;
+            U32 reserved        : 18;
+        };
+    };
+} DSPARB_Register, *PDSPARB_Register;
+
+typedef enum CePlane_t
+{
+    DCplane_A,
+    DCplane_B,
+    DCplane_C,
+    DCplane_CursorA,
+    DCplane_CursorB,
+    DCplane_Overlay1,
+    DC_NUM_PLANES,
+    DCplane_NotSupported = -1
+} CePlane;
+
+typedef struct _Watermark_Register1
+{
+    union
+    {
+        U32 entireRegister;
+        struct
+        {
+            U32 DispA_WM1	: 8;
+            U32 DispA_BL	: 2;
+            U32 reserved	: 6;
+            U32 DispB_WM1	: 8;
+            U32 DispB_BL	: 2;
+            U32 DispC_WM1	: 5;
+            U32 reserved1	: 1;
+        };
+    };
+} Watermark_Register1, *PWatermark_Register1;
+
+typedef struct _FIFOWatermark_Register
+{
+    union
+    {
+        U32 entireRegister;
+        struct
+        {
+            U32 DispAB_WM2		: 8;
+            U32 CursAB_WM2		: 7;
+            U32 maxFIFO_Enable		: 1;
+            U32 reserved		: 1;
+            U32 OverlayUV_WM2		: 7;
+            U32 OverlayY_WM2		: 7;
+            U32 reserved1		: 1;
+        };
+    };
+} FIFOWatermark_Register, *PFIFOWatermark_Register;
+
+typedef struct _MAXFIFOWatermark_Register
+{
+    union
+    {
+        U32 entireRegister;
+        struct
+        {
+            U32 WM1_STATUS1		: 8;
+            U32 WM1_STATUS0		: 8;
+            U32 reserved		: 16;
+        };
+    };
+} MAXFIFOWatermark_Register, *PMAXFIFOWatermark_Register;
+
+
 static inline PsbPtr
 psbPTR(ScrnInfoPtr pScrn)
 {
