@@ -609,7 +609,7 @@ psbDisplayVideo(ScrnInfoPtr pScrn, PsbPortPrivPtr pPriv, int id,
     XpsbSurfacePtr src[3];
     float tc0[6], tc1[6], tc2[6];
     int num_texture = 0;
-    float *conversion_data = NULL;
+    float *conversion_data = NULL;    
 
     hdtv = ((src_w >= PSB_HDTV_LIMIT_X) && (src_h >= PSB_HDTV_LIMIT_Y));
 
@@ -715,6 +715,12 @@ psbDisplayVideo(ScrnInfoPtr pScrn, PsbPortPrivPtr pPriv, int id,
     pbox = REGION_RECTS(dstRegion);
     nbox = REGION_NUM_RECTS(dstRegion);
 
+#ifdef PSB_DETEAR
+    /* we don't support multiple box render detearing with temp buffer
+       very well, have to fall back to original rendering */
+    int fallback = (nbox>1);
+#endif	/* PSB_DETEAR */
+
     while (nbox--) {
 	int box_x1 = pbox->x1;
 	int box_y1 = pbox->y1;
@@ -737,14 +743,34 @@ psbDisplayVideo(ScrnInfoPtr pScrn, PsbPortPrivPtr pPriv, int id,
 
 	pbox++;
 
-	psbBlitYUV(pScrn,
-		   &dst,
-		   &src[0],
-		   num_texture,
-		   TRUE,
-		   src[0]->packedYUVId,
-		   tc0, tc1, tc2, num_texture, conversion_data);
+#ifdef PSB_DETEAR
+	PsbPtr pPsb = psbPTR(pScrn);
+
+	if (pPsb->vsync && !fallback) {
+		psbBlitYUVDetear(pScrn,
+				 &dst,
+				 &src[0],
+				 num_texture,
+				 TRUE,
+				 src[0]->packedYUVId,
+				 tc0, tc1, tc2, 
+				 num_texture, 
+				 conversion_data);
+	} else
+#endif	/* PSB_DETEAR */
+		{
+			psbBlitYUV(pScrn,
+				   &dst,
+				   &src[0],
+				   num_texture,
+				   TRUE,
+				   src[0]->packedYUVId,
+				   tc0, tc1, tc2, 
+				   num_texture, 
+				   conversion_data);			
+		} 
     }
+
     DamageDamageRegion(&pPixmap->drawable, dstRegion);
     return TRUE;
 }
@@ -859,6 +885,7 @@ psbPutImage(ScrnInfoPtr pScrn,
 	pPixmap = (PixmapPtr) pDraw;
     }
 
+
     psbDisplayVideo(pScrn, pPriv, destId, clipBoxes, width, height,
 		    dstPitch, x1, y1, x2, y2,
 		    src_w, src_h, drw_w, drw_h, pPixmap);
